package com.appcarry.appcarry_sdk;

import android.app.Activity;
import android.app.Application;
import android.content.Context;
import android.content.Intent;
import android.os.Handler;
import android.os.Looper;
import android.util.Log;

import com.appcarry.sdk.Appcarry;
import com.appcarry.sdk.AppcarryProperties;
import com.appcarry.sdk.deeplink.DeepLinkListener;
import com.appcarry.sdk.interfaces.AppCarryConversionListener;
import com.appcarry.sdk.linktrack.CreateLinkTrackTask;
import com.appcarry.sdk.share.CrossPromotion;
import com.appcarry.sdk.share.ShareInviteHelper;
import com.appcarry.sdk.share.ShareLinkGenerator;

import org.json.JSONException;
import org.json.JSONObject;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.BinaryMessenger;
import io.flutter.plugin.common.EventChannel;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;

import static com.appcarry.appcarry_sdk.AppcarryConstants.AC_EVENTS_CHANNEL;
import static com.appcarry.appcarry_sdk.AppcarryConstants.AC_FAILURE;
import static com.appcarry.appcarry_sdk.AppcarryConstants.AC_ON_APP_OPEN_ATTRIBUTION;
import static com.appcarry.appcarry_sdk.AppcarryConstants.AC_ON_INSTALL_CONVERSION_DATA_LOADED;
import static com.appcarry.appcarry_sdk.AppcarryConstants.AC_SUCCESS;

/**
 * AppcarrySdkPlugin
 */
public class AppcarrySdkPlugin implements FlutterPlugin, MethodCallHandler, ActivityAware {

    final Handler uiThreadHandler = new Handler(Looper.getMainLooper());
    private EventChannel mEventChannel;
    /**
     * Plugin registration.
     */
    //private FlutterView mFlutterView;
    private Context mContext;
    private Application mApplication;
    private Intent mIntent;
    private MethodChannel mMethodChannel;
    private MethodChannel mCallbackChannel;
    private Activity activity;
    private Boolean gcdCallback = false;
    private Boolean oaoaCallback = false;
    private Boolean udlCallback = false;
    private Boolean validatePurchaseCallback = false;
    private Boolean isFacebookDeferredApplinksEnabled = false;
    private Map<String, Map<String, Object>> mCallbacks = new HashMap<>();
    MethodCallHandler callbacksHandler = new MethodCallHandler() {
        @Override
        public void onMethodCall(MethodCall call, Result result) {
            final String method = call.method;
            if ("startListening".equals(method)) {
                startListening(call.arguments, result);
            } else {
                result.notImplemented();
            }
        }
    };

    private void onAttachedToEngine(Context applicationContext, BinaryMessenger messenger) {
        this.mContext = applicationContext;
        this.mEventChannel = new EventChannel(messenger, AC_EVENTS_CHANNEL);
        mEventChannel.setStreamHandler(new AppCarryStreamHandler(mContext));
        mMethodChannel = new MethodChannel(messenger, AppcarryConstants.AC_METHOD_CHANNEL);
        mMethodChannel.setMethodCallHandler(this);
        mCallbackChannel = new MethodChannel(messenger, AppcarryConstants.AC_CALLBACK_CHANNEL);
        mCallbackChannel.setMethodCallHandler(callbacksHandler);
    }

    private void startListening(Object arguments, Result rawResult) {
        // Get callback id
        String callbackName = (String) arguments;
        if (callbackName.equals(AppcarryConstants.AC_GCD_CALLBACK)) {
            gcdCallback = true;
        }
        if (callbackName.equals(AppcarryConstants.AC_OAOA_CALLBACK)) {
            oaoaCallback = true;
        }
        if (callbackName.equals(AppcarryConstants.AC_UDL_CALLBACK)) {
            udlCallback = true;
        }
        if (callbackName.equals(AppcarryConstants.AC_VALIDATE_PURCHASE)) {
            validatePurchaseCallback = true;
        }
        Map<String, Object> args = new HashMap<>();
        args.put("id", callbackName);
        mCallbacks.put(callbackName, args);

        rawResult.success(null);
    }

    @Override
    public void onMethodCall(MethodCall call, Result result) {
        final String method = call.method;
        switch (method) {
            case "initSdk":
                initSdk(call, result);
                break;
            case "logEvent":
                logEvent(call, result);
                break;
            case "setHost":
                setHost(call, result);
                break;
            case "setCurrencyCode":
                setCurrencyCode(call, result);
                break;
            case "setIsUpdate":
                setIsUpdate(call, result);
                break;
            case "stop":
                stop(call, result);
                break;
            case "updateServerUninstallToken":
                updateServerUninstallToken(call, result);
                break;
            case "setImeiData":
                setImeiData(call, result);
                break;
            case "setAndroidIdData":
                setAndroidIdData(call, result);
                break;
            case "enableLocationCollection":
                enableLocationCollection(call, result);
                break;
            case "setCustomerUserId":
                setCustomerUserId(call, result);
                break;
            case "waitForCustomerUserId":
                waitForCustomerUserId(call, result);
                break;
            case "setAdditionalData":
                setAdditionalData(call, result);
                break;
            case "setUserEmails":
                setUserEmails(call, result);
                break;
            case "setUserEmailsWithCryptType":
                setUserEmailsWithCryptType(call, result);
                break;
            case "setCollectAndroidId":
                setCollectAndroidId(call, result);
                break;
            case "setCollectIMEI":
                setCollectIMEI(call, result);
                break;
            case "getHostName":
                getHostName(result);
                break;
            case "getHostPrefix":
                getHostPrefix(result);
                break;
            case "setMinTimeBetweenSessions":
                setMinTimeBetweenSessions(call, result);
                break;
            case "validateAndLogInAppAndroidPurchase":
                validateAndLogInAppPurchase(call, result);
                break;
            case "getAppsCarryUID":
                getAppCarryUID(result);
                break;
            case "getSDKVersion":
                getSdkVersion(result);
                break;
            case "setSharingFilter":
                setSharingFilter(call, result);
                break;
            case "setSharingFilterForAllPartners":
                setSharingFilterForAllPartners(result);
                break;
            case "generateInviteLink":
                generateInviteLink(call, result);
                break;
            case "setAppInviteOneLinkID":
                setAppInivteOneLinkID(call, result);
                break;
            case "logCrossPromotionImpression":
                logCrossPromotionImpression(call, result);
                break;
            case "logCrossPromotionAndOpenStore":
                logCrossPromotionAndOpenStore(call, result);
                break;
            case "setLinkTrackCustomDomain":
                setLinkTrackCustomDomain(call, result);
                break;
            case "setPushNotification":
                sendPushNotificationData(call, result);
                break;
            case "enableFacebookDeferredApplinks":
                enableFacebookDeferredApplinks(call, result);
                break;
            case "setWebHookURL":
                setWebHookURL(call, result);
                break;
            default:
                result.notImplemented();
                break;
        }
    }

    private void enableFacebookDeferredApplinks(MethodCall call, Result result) {
        isFacebookDeferredApplinksEnabled = call.argument("isFacebookDeferredApplinksEnabled");
        if (isFacebookDeferredApplinksEnabled) {
            Appcarry.getInstance().enableFacebookDeferredApplinks(true);
        } else {
            Appcarry.getInstance().enableFacebookDeferredApplinks(false);
        }
        result.success(null);
    }

    private void sendPushNotificationData(MethodCall call, Result result) {
        Appcarry.getInstance().sendPushNotificationData(activity);
        result.success(null);
    }

    private void setLinkTrackCustomDomain(MethodCall call, Result result) {
        ArrayList<String> brandDomains = (ArrayList<String>) call.arguments;
        String[] brandDomainsArray = brandDomains.toArray(new String[brandDomains.size()]);
        Appcarry.getInstance().setLinkTrackCustomDomain(brandDomainsArray);
        result.success(null);
    }

    private void logCrossPromotionAndOpenStore(MethodCall call, Result result) {
        String appId = call.argument("appId");
        String campaign = call.argument("campaign");
        Map data = (Map) call.argument("params");
        if (appId != null && !appId.equals("")) {
            CrossPromotion.logAndOpenStore(mContext, appId, campaign, data);
        }
        result.success(null);
    }

    private void logCrossPromotionImpression(MethodCall call, Result result) {
        String appId = call.argument("appId");
        String campaign = call.argument("campaign");
        Map data = (Map) call.argument("data");
        if (appId != null && !appId.equals("")) {
            CrossPromotion.logCrossPromoteImpression(mContext, appId, campaign, data);
        }
        result.success(null);
    }

    private void setAppInviteLinkTrack(MethodCall call, Result result) {
        String inviteLinkTrackID = call.argument("inviteLinkTrackID");
        if (inviteLinkTrackID == null || inviteLinkTrackID.length() == 0) {
            result.success(null);
        } else {
            Appcarry.getInstance().setAppInviteLinkTrack(inviteLinkTrackID);
            if (mCallbacks.containsKey("setAppInviteLinkTrackCallback")) {
                JSONObject obj = buildJsonResponse("success", AC_SUCCESS);
                runOnUIThread(obj, "setAppInviteLinkTrackCallback", AC_SUCCESS);
            }
        }
    }

    private void generateInviteLink(MethodCall call, Result rawResult) {
        String channel = call.argument("channel");
        String customerID = call.argument("customerID");
        String campaign = call.argument("campaign");
        String referrerName = call.argument("referrerName");
        String referrerImageUrl = call.argument("referrerImageUrl");
        String baseDeepLink = call.argument("baseDeeplink");
        String brandDomain = call.argument("brandDomain");

        ShareLinkGenerator linkGenerator = ShareInviteHelper.generateInviteUrl(mContext);

        if (channel != null && !channel.equals("")) {
            linkGenerator.setChannel(channel);
        }
        if (campaign != null && !campaign.equals("")) {
            linkGenerator.setCampaignName(campaign);
        }
        if (referrerName != null && !referrerName.equals("")) {
            linkGenerator.setReferrerName(referrerName);
        }
        if (referrerImageUrl != null && !referrerImageUrl.equals("")) {
            linkGenerator.setReferrerImageURL(referrerImageUrl);
        }
        if (customerID != null && !customerID.equals("")) {
            linkGenerator.setReferrerCustomerId(customerID);
        }
        if (baseDeepLink != null && !baseDeepLink.equals("")) {
            linkGenerator.setBaseDeepLink(baseDeepLink);
        }
        if (brandDomain != null && !brandDomain.equals("")) {
            linkGenerator.setBrandDomain(brandDomain);
        }

        CreateLinkTrackTask.a listener = new CreateLinkTrackTask.a() {
            JSONObject obj = new JSONObject();

            @Override
            public void a(String s) {
                if (mCallbacks.containsKey("generateInviteLinkSuccess")) {
                    try {
                        obj.put("userInviteUrl", s);
                        runOnUIThread(obj, "generateInviteLinkSuccess", AC_SUCCESS);
                    } catch (JSONException e) {
                        e.printStackTrace();
                    }
                }
            }

            @Override
            public void b(String s) {
                if (mCallbacks.containsKey("generateInviteLinkFailure")) {
                    try {
                        obj.put("error", s);
                        runOnUIThread(s, "generateInviteLinkFailure", AC_FAILURE);
                    } catch (JSONException e) {
                        e.printStackTrace();
                    }
                }
            }
        };

        linkGenerator.generateLink(mContext, listener);

        rawResult.success(null);
    }

    private void runOnUIThread(final Object data, final String callbackName, final String status) {
        uiThreadHandler.post(
                () -> {
                    Log.d("Callbacks", "Calling invokeMethod with: " + data);
                    JSONObject args = new JSONObject();
                    try {
                        args.put("id", callbackName);
                        args.put("status", status);
                        args.put("data", data.toString());
                    } catch (JSONException e) {
                        e.printStackTrace();
                    }
                    mCallbackChannel.invokeMethod("callListener", args.toString());
                }
        );
    }

    private void setSharingFilterForAllPartners(Result result) {
        Appcarry.getInstance().setSharingFilterForAllPartners();
        result.success(null);
    }

    private void setSharingFilter(MethodCall call, Result result) {
        Appcarry.getInstance().setSharingFilter();
        result.success(null);
    }

    private void getAppCarryUID(Result result) {
        result.success(Appcarry.getInstance().getAppcarryUID(this.mContext));
    }

    private void setUserEmailsWithCryptType(MethodCall call, Result result) {
        List<String> emails = call.argument("emails");
        int cryptTypeInt = call.argument("cryptType");
        AppcarryProperties.a cryptType = AppcarryProperties.a.values()[cryptTypeInt];
        if (emails != null) {
            Appcarry.getInstance().setUserEmails(cryptType, emails.toArray(new String[0]));
        }
    }

    private void validateAndLogInAppPurchase(MethodCall call, Result result) {
//    registerValidatorListener();
        String publicKey = call.argument("publicKey");
        String signature = call.argument("signature");
        String purchaseData = call.argument("purchaseData");
        String price = call.argument("price");
        String currency = call.argument("currency");
        Map<String, String> additionalParameters = call.argument("additionalParameters");
        Appcarry.getInstance().validateAndLogInAppPurchase(mContext, publicKey, signature, purchaseData, price,
                currency, additionalParameters);
        result.success(null);
    }

//  private void registerValidatorListener() {
//    AppsFlyerInAppPurchaseValidatorListener validatorListener = new AppsFlyerInAppPurchaseValidatorListener() {
//      @Override
//      public void onValidateInApp() {
//        try {
//          JSONObject obj = new JSONObject();
//          if(validatePurchaseCallback){
//            runOnUIThread(obj, AppcarryConstants.AF_VALIDATE_PURCHASE, AF_SUCCESS);
//          }else{
//            obj.put("status", AF_SUCCESS);
//            sendEventToDart(obj, AF_EVENTS_CHANNEL);
//          }
//        } catch (JSONException e) {
//          e.printStackTrace();
//        }
//
//      }
//
//      @Override
//      public void onValidateInAppFailure(String s) {
//        try {
//          JSONObject obj = new JSONObject();
//          obj.put("error", s);
//          if(validatePurchaseCallback){
//            runOnUIThread(obj, AppcarryConstants.AF_VALIDATE_PURCHASE, AF_FAILURE);
//          }else{
//            obj.put("status", AF_FAILURE);
//            sendEventToDart(obj, AF_EVENTS_CHANNEL);
//          }
//        } catch (JSONException e) {
//          e.printStackTrace();
//        }
//      }
//    };
//    Appcarry.getInstance().registerValidatorListener(mContext, validatorListener);
//  }

    private void setMinTimeBetweenSessions(MethodCall call, Result result) {
        int seconds = call.argument("seconds");
        Appcarry.getInstance().setMinTimeBetweenSessions(seconds);
        result.success(null);
    }

    private void getHostPrefix(Result result) {
        result.success(Appcarry.getInstance().getHostPrefix());
    }

    private void getSdkVersion(Result result) {
        result.success(Appcarry.getInstance().getSdkVersion());
    }

    private void getHostName(Result result) {
        result.success(Appcarry.getInstance().getHostName());
    }

    private void setCollectIMEI(MethodCall call, Result result) {
        boolean isCollect = call.argument("isCollect");
        Appcarry.getInstance().setCollectIMEI(isCollect);
        result.success(null);
    }

    private void setCollectAndroidId(MethodCall call, Result result) {
        boolean isCollect = call.argument("isCollect");
        Appcarry.getInstance().setCollectAndroidID(isCollect);
        result.success(null);
    }

    private void waitForCustomerUserId(MethodCall call, Result result) {
        boolean wait = call.argument("wait");
        Appcarry.getInstance().waitForCustomerUserId(wait);
        result.success(null);
    }

    private void setAdditionalData(MethodCall call, Result result) {
        HashMap<String, Object> customData = call.argument("customData");
        Appcarry.getInstance().setAdditionalData(customData);
        result.success(null);
    }

    private void setUserEmails(MethodCall call, Result result) {
        List<String> userEmails = call.argument("emails");
        if (userEmails != null) {
            Appcarry.getInstance().setUserEmails(userEmails.toArray(new String[0]));
        }
        result.success(null);
    }

    private void setCustomerUserId(MethodCall call, Result result) {
        String userId = call.argument("id");
        Appcarry.getInstance().setCustomerUserId(userId);
        result.success(null);
    }

    private void enableLocationCollection(MethodCall call, Result result) {
        boolean flag = call.argument("flag");
        Appcarry.getInstance().enableLocationCollection(flag);
        result.success(null);
    }

    private void setAndroidIdData(MethodCall call, Result result) {
        String androidId = call.argument("androidId");
        Appcarry.getInstance().setAndroidIdData(androidId);
        result.success(null);
    }

    private void setImeiData(MethodCall call, Result result) {
        String imei = call.argument("imei");
        Appcarry.getInstance().setImeiData(imei);
        result.success(null);
    }

    private void updateServerUninstallToken(MethodCall call, Result result) {
        String token = call.argument("token");
        Appcarry.getInstance().updateServerUninstallToken(mContext, token);
        result.success(null);
    }

    private void stop(MethodCall call, Result result) {
        boolean isStopped = call.argument("isStopped");
        Appcarry.getInstance().stop(isStopped, mContext);
        result.success(null);
    }

    private void setIsUpdate(MethodCall call, Result result) {
        boolean isUpdate = call.argument("isUpdate");
        Appcarry.getInstance().setIsUpdate(isUpdate);
        result.success(null);
    }

    private void setCurrencyCode(MethodCall call, Result result) {
        String currencyCode = call.argument("currencyCode");
        Appcarry.getInstance().setCurrencyCode(currencyCode);
        result.success(null);
    }

    private void setWebHookURL(MethodCall call, Result result) {
        String webHookURL = call.argument("webHookURL");
        Appcarry.getInstance().setWebHookURL(webHookURL);
        result.success(null);
    }

    private void setHost(MethodCall call, MethodChannel.Result result) {
        String hostPrefix = call.argument(AppcarryConstants.AC_HOST_PREFIX);
        String hostName = call.argument(AppcarryConstants.AC_HOST_NAME);

        Appcarry.getInstance().setHost(hostPrefix, hostName);
    }

    private void initSdk(MethodCall call, final MethodChannel.Result result) {
        AppCarryConversionListener gcdListener = null;
        DeepLinkListener udlListener = null;
        Appcarry instance = Appcarry.getInstance();

        String acDevKey = call.argument(AppcarryConstants.AC_DEV_KEY);
        if (acDevKey == null || acDevKey.equals("")) {
            result.error(null, "AC Dev Key is empty", "AC dev key cannot be empty");
        }

        boolean getGCD = call.argument(AppcarryConstants.AC_GCD);
        if (getGCD) {
            gcdListener = registerConversionListener();
        }
        // added Unified deeplink
//        boolean getUdl = call.argument(AppcarryConstants.AC_UDL) ;
//        if (getUdl) {
//            udlListener = registerOnDeeplinkingListener();
//            instance.subscribeForDeepLink(udlListener);
//        }

        boolean isDebug = call.argument(AppcarryConstants.AC_IS_DEBUG);
        if (isDebug) {
//      instance.setLogLevel(AFLogger.LogLevel.DEBUG);
            instance.setDebugLog(true);
        } else {
            instance.setDebugLog(false);
        }

        instance.init(acDevKey, gcdListener, mContext);

        String appInviteOneLink = call.argument(AppcarryConstants.AC_APP_INVITE_ONE_LINK);
        if (appInviteOneLink != null) {
            instance.setAppInviteLinkTrack(appInviteOneLink);
        }

        instance.start(activity);

        result.success("success");
    }

    private void logEvent(MethodCall call, MethodChannel.Result result) {

        Appcarry instance = Appcarry.getInstance();

        final String eventName = call.argument(AppcarryConstants.AC_EVENT_NAME);
        final Map<String, Object> eventValues = call.argument(AppcarryConstants.AC_EVENT_VALUES);

        // Send event data through appcarry sdk
        instance.logEvent(mContext, eventName, eventValues);

        result.success(true);
    }

    private DeepLinkListener registerOnDeeplinkingListener() {
        return deepLinkResult -> {
            if (udlCallback) {
                runOnUIThread(deepLinkResult, AppcarryConstants.AC_UDL_CALLBACK, AC_SUCCESS);
            } else {
                try {
                    JSONObject obj = new JSONObject();
                    obj.put("status", AC_SUCCESS);
                    obj.put("type", AppcarryConstants.AC_UDL_CALLBACK);
//                        obj.put("data", deepLinkResult.getDeepLink().getClickEvent());

                    sendEventToDart(obj, AC_EVENTS_CHANNEL);
                } catch (JSONException e) {
                    e.printStackTrace();
                }
            }
        };
    }

    private AppCarryConversionListener registerConversionListener() {
        return new AppCarryConversionListener() {
            @Override
            public void onConversionDataSuccess(Map<String, Object> map) {
                if (gcdCallback) {
                    JSONObject dataObj = new JSONObject(replaceNullValues(map));
                    runOnUIThread(dataObj, AppcarryConstants.AC_GCD_CALLBACK, AC_SUCCESS);
                } else {
                    handleSuccess(AC_ON_INSTALL_CONVERSION_DATA_LOADED, map, AC_EVENTS_CHANNEL);
                }
            }

            @Override
            public void onConversionDataFail(String s) {
                if (gcdCallback) {
                    JSONObject obj = buildJsonResponse(s, AC_FAILURE);
                    runOnUIThread(obj, AppcarryConstants.AC_GCD_CALLBACK, AC_FAILURE);
                } else {
                    handleError(AC_ON_INSTALL_CONVERSION_DATA_LOADED, s, AC_EVENTS_CHANNEL);
                }
            }

            @Override
            public void onAppOpenAttribution(Map<String, Object> map) {
                Map<String, Object> objMap = (Map) map;
                if (oaoaCallback) {
                    JSONObject obj = new JSONObject(replaceNullValues(objMap));
                    runOnUIThread(obj, AppcarryConstants.AC_OAOA_CALLBACK, AC_SUCCESS);
                } else {
                    handleSuccess(AC_ON_APP_OPEN_ATTRIBUTION, objMap, AC_EVENTS_CHANNEL);
                }
            }

            @Override
            public void onAttributionFailure(String errorMessage) {
                if (oaoaCallback) {
                    JSONObject obj = buildJsonResponse(errorMessage, AC_FAILURE);
                    runOnUIThread(obj, AppcarryConstants.AC_OAOA_CALLBACK, AC_FAILURE);
                } else {
                    handleError(AC_ON_APP_OPEN_ATTRIBUTION, errorMessage, AC_EVENTS_CHANNEL);
                }
            }
        };
    }

    private void handleSuccess(String eventType, Map<String, Object> data, String channel) {
        try {
            JSONObject obj = new JSONObject();
            obj.put("status", AC_SUCCESS);
            obj.put("type", eventType);
            obj.put("data", new JSONObject(replaceNullValues(data)));

            sendEventToDart(obj, channel);
        } catch (JSONException e) {
            e.printStackTrace();
        }
    }

    private void handleError(String eventType, String errorMessage, String channel) {

        try {
            JSONObject obj = new JSONObject();

            obj.put("status", AC_FAILURE);
            obj.put("type", eventType);
            obj.put("data", errorMessage);

            sendEventToDart(obj, channel);
        } catch (JSONException e) {
            e.printStackTrace();
        }
    }

    private JSONObject buildJsonResponse(Object data, String status) {
        JSONObject obj = new JSONObject();
        try {
            obj.put("status", status);
            obj.put("data", data.toString());
        } catch (JSONException e) {
            e.printStackTrace();
        }
        return obj;
    }

    private Map<String, Object> replaceNullValues(Map<String, Object> map) {
        // cant use stream because of older versions of java
        Map<String, Object> newMap = new HashMap<
                >();
        Iterator it = map.entrySet().iterator();
        while (it.hasNext()) {
            Map.Entry<String, Object> pair = (Map.Entry) it.next();
            newMap.put(pair.getKey(), pair.getValue() == null ? JSONObject.NULL : pair.getValue());
            it.remove(); // avoids a ConcurrentModificationException
        }

        return newMap;
    }

    private void sendEventToDart(final JSONObject params, String channel) {
        Intent intent = new Intent();
        intent.addFlags(Intent.FLAG_INCLUDE_STOPPED_PACKAGES);
        intent.setAction(AppcarryConstants.AC_BROADCAST_ACTION_NAME);
        intent.putExtra("params", params.toString());
        mContext.sendBroadcast(intent);
    }

    @Override
    public void onAttachedToEngine(FlutterPluginBinding binding) {
        onAttachedToEngine(binding.getApplicationContext(), binding.getBinaryMessenger());
    }

    @Override
    public void onDetachedFromEngine(FlutterPluginBinding binding) {
        mMethodChannel.setMethodCallHandler(null);
        mMethodChannel = null;
        mEventChannel.setStreamHandler(null);
        mEventChannel = null;
    }

    @Override
    public void onAttachedToActivity(ActivityPluginBinding binding) {
        activity = binding.getActivity();
        mIntent = binding.getActivity().getIntent();
        mApplication = binding.getActivity().getApplication();
    }

    @Override
    public void onDetachedFromActivityForConfigChanges() {

    }

    @Override
    public void onReattachedToActivityForConfigChanges(ActivityPluginBinding binding) {

    }

    @Override
    public void onDetachedFromActivity() {
        activity = null;
    }
}