part of appcarry_sdk;

class AppCarrySdk {
  StreamController _acGCDStreamController;
  StreamController _acOpenAttributionStreamController;
  StreamController _acValidtaPurchaseController;
  EventChannel _eventChannel;
  static AppCarrySdk _instance;
  final MethodChannel _methodChannel;

  AppCarryOptions acOptions;
  Map mapOptions;

  ///Returns the [AppCarrySdk] instance, initialized with a custom options
  ///provided by the user
  factory AppCarrySdk(options) {
    if (_instance == null) {
      MethodChannel methodChannel =
          const MethodChannel(AppCarryConstants.AC_METHOD_CHANNEL);

      EventChannel eventChannel =
          EventChannel(AppCarryConstants.AC_EVENTS_CHANNEL);

      //check if the option variable is AFOptions type or map type
      if (options is AppCarryOptions) {
        _instance = AppCarrySdk.private(methodChannel, eventChannel,
            acOptions: options);
      } else if (options is Map) {
        _instance = AppCarrySdk.private(methodChannel, eventChannel,
            mapOptions: options);
      }
    }
    return _instance;
  }

  @visibleForTesting
  AppCarrySdk.private(this._methodChannel, this._eventChannel,
      {this.acOptions, this.mapOptions});

  Map<String, dynamic> _validateACOptions(AppCarryOptions options) {
    Map<String, dynamic> validatedOptions = {};

    //validations
    dynamic devKey = options.acDevKey;
    assert(devKey != null);
    assert(devKey is String);

    validatedOptions[AppCarryConstants.AC_DEV_KEY] = devKey;

    dynamic appInviteOneLink = options.appInviteLinkTrack;
    if (appInviteOneLink != null) {
      assert(appInviteOneLink is String);
    }

    validatedOptions[AppCarryConstants.APP_INVITE_ONE_LINK] = appInviteOneLink;

    if (options.disableCollectASA != null) {
      validatedOptions[AppCarryConstants.DISABLE_COLLECT_ASA] =
          options.disableCollectASA;
    }

    if (options.disableAdvertisingIdentifier != null) {
      validatedOptions[AppCarryConstants.DISABLE_ADVERTISING_IDENTIFIER] =
          options.disableAdvertisingIdentifier;
    }

    if (Platform.isIOS) {
      dynamic appID = options.appId;
      assert(appID != null, "appleAppId is required for iOS apps");
      assert(appID is String);
      RegExp exp = RegExp(r'^\d{8,11}$');
      assert(exp.hasMatch(appID));
      validatedOptions[AppCarryConstants.AC_APP_ID] = appID;
    }

    validatedOptions[AppCarryConstants.AC_IS_DEBUG] =
        (options.showDebug != null) ? options.showDebug : false;

    if (_acGCDStreamController != null ||
        _acOpenAttributionStreamController != null) {
      validatedOptions[AppCarryConstants.AC_GCD] = true;
    } else {
      validatedOptions[AppCarryConstants.AC_GCD] = false;
    }

    return validatedOptions;
  }

  Map<String, dynamic> _validateMapOptions(Map options) {
    Map<String, dynamic> acOptions = {};
    //validations
    dynamic devKey = options[AppCarryConstants.AC_DEV_KEY];
    assert(devKey != null);
    assert(devKey is String);

    acOptions[AppCarryConstants.AC_DEV_KEY] = devKey;

    dynamic appInviteLinkTrack = options[AppCarryConstants.APP_INVITE_ONE_LINK];
    if (appInviteLinkTrack != null) {
      assert(appInviteLinkTrack is String);
    }

    acOptions[AppCarryConstants.APP_INVITE_ONE_LINK] = appInviteLinkTrack;

    if (options[AppCarryConstants.DISABLE_COLLECT_ASA] != null) {
      acOptions[AppCarryConstants.DISABLE_COLLECT_ASA] =
          options[AppCarryConstants.DISABLE_COLLECT_ASA];
    }

    if (options[AppCarryConstants.DISABLE_ADVERTISING_IDENTIFIER] != null) {
      acOptions[AppCarryConstants.DISABLE_ADVERTISING_IDENTIFIER] =
          options[AppCarryConstants.DISABLE_ADVERTISING_IDENTIFIER];
    }

    if (Platform.isIOS) {
      if (options[
              AppCarryConstants.AC_TIME_TO_WAIT_FOR_ATT_USER_AUTHORIZATION] !=
          null) {
        dynamic timeToWaitForATTUserAuthorization = options[
            AppCarryConstants.AC_TIME_TO_WAIT_FOR_ATT_USER_AUTHORIZATION];
        assert(timeToWaitForATTUserAuthorization is double);

        acOptions[
                AppCarryConstants.AC_TIME_TO_WAIT_FOR_ATT_USER_AUTHORIZATION] =
            timeToWaitForATTUserAuthorization;
      }

      dynamic appID = options[AppCarryConstants.AC_APP_ID];
      assert(appID != null, "appleAppId is required for iOS apps");
      assert(appID is String);
      RegExp exp = RegExp(r'^\d{8,11}$');
      assert(exp.hasMatch(appID));
      acOptions[AppCarryConstants.AC_APP_ID] = appID;
    }

    acOptions[AppCarryConstants.AC_IS_DEBUG] =
        options.containsKey(AppCarryConstants.AC_IS_DEBUG)
            ? options[AppCarryConstants.AC_IS_DEBUG]
            : false;

    if (_acGCDStreamController != null ||
        _acOpenAttributionStreamController != null) {
      acOptions[AppCarryConstants.AC_GCD] = true;
    } else {
      acOptions[AppCarryConstants.AC_GCD] = false;
    }

    return acOptions;
  }

  // Accessing AppCarry Conversion Data from the SDK
  void _registerConversionDataCallback() {
    if (_acGCDStreamController == null) {
      _acGCDStreamController = StreamController<Map>(onCancel: () {
        _acGCDStreamController.close();
      });
    }
  }

  Stream<Map> get conversionDataStream {
    return _acGCDStreamController?.stream?.asBroadcastStream();
  }

  // Accessing AppCarry attribution, referred from deep linking
  void _registerOnAppOpenAttributionCallback() {
    if (_acOpenAttributionStreamController == null) {
      _acOpenAttributionStreamController = StreamController<Map>(onCancel: () {
        _acOpenAttributionStreamController.close();
      });
    }
  }

  Stream<Map> get appOpenAttributionStream {
    return _acOpenAttributionStreamController?.stream?.asBroadcastStream();
  }

  ///Returns `Stream`. Accessing AppCarry purchase validation data
  Stream<dynamic> _registerValidatePurchaseCallback() {
    if (_acValidtaPurchaseController == null) {
      _acValidtaPurchaseController = StreamController(onCancel: () {
        _acValidtaPurchaseController.close();
      });

      _registerPurchaseValidateListener();
    }
    return _acValidtaPurchaseController.stream;
  }

  ///initialize the SDK, using the options initialized from the constructor|
  Future<dynamic> initSdk(
      {bool registerConversionDataCallback = false,
      bool registerOnAppOpenAttributionCallback = false}) async {
    return Future.delayed(Duration(seconds: 0)).then((_) {
      if (registerConversionDataCallback) _registerConversionDataCallback();
      if (registerOnAppOpenAttributionCallback)
        _registerOnAppOpenAttributionCallback();

      if (registerConversionDataCallback ||
          registerOnAppOpenAttributionCallback) {
        _registerGCDListener();
      }

      Map<String, dynamic> validatedOptions;
      if (mapOptions != null) {
        validatedOptions = _validateMapOptions(mapOptions);
      } else if (acOptions != null) {
        validatedOptions = _validateACOptions(acOptions);
      }

      return _methodChannel.invokeMethod("initSdk", validatedOptions);
    });
  }

  Future<String> getSDKVersion() async {
    return _methodChannel.invokeMethod("getSDKVersion");
  }

  ///These in-app events help you to log how loyal users discover your app, and attribute them to specific
  ///campaigns/media-sources. Please take the time define the event/s you want to measure to allow you
  ///to send ROI (Return on Investment) and LTV (Lifetime Value).
  ///- The `logEvent` method allows you to send in-app events to AppCarry analytics. This method allows you to add events dynamically by adding them directly to the application code.
  Future<bool> logEvent(String eventName, Map eventValues) async {
    assert(eventValues != null);

    return await _methodChannel.invokeMethod(
        "logEvent", {'eventName': eventName, 'eventValues': eventValues});
  }

  void setHost(String hostPrefix, String hostName) {
    _methodChannel.invokeMethod(
        "setHost", {'hostPrefix': hostPrefix, 'hostName': hostName});
  }

  /// Opt-out of collection of IMEI.
  /// If the app does NOT contain Google Play Services, device IMEI is collected by the SDK.
  /// However, apps with Google play services should avoid IMEI collection as this is in violation of the Google Play policy.
  void setCollectIMEI(bool isCollect) {
    _methodChannel.invokeMethod("setCollectIMEI", {'isCollect': isCollect});
  }

  /// Opt-out of collection of Android ID.
  /// If the app does NOT contain Google Play Services, Android ID is collected by the SDK.
  /// However, apps with Google play services should avoid Android ID collection as this is in violation of the Google Play policy.
  void setCollectAndroidId(bool isCollect) {
    _methodChannel
        .invokeMethod("setCollectAndroidId", {'isCollect': isCollect});
  }

  Future<String> getHostName() async {
    return await _methodChannel.invokeMethod("getHostName");
  }

  Future<String> getHostPrefix() async {
    return await _methodChannel.invokeMethod("getHostPrefix");
  }

  void setAndroidIdData(String androidId) {
    _methodChannel.invokeMethod("setAndroidIdData", {'androidId': androidId});
  }

  void setMinTimeBetweenSessions(int seconds) {
    assert(seconds >= 0, "the minimum timeout must be a positive number");
    _methodChannel
        .invokeMethod("setMinTimeBetweenSessions", {'seconds': seconds});
  }

  void setImeiData(String imei) {
    _methodChannel.invokeMethod("setImeiData", {'imei': imei});
  }

  /// Setting user local currency code for in-app purchases.
  /// The currency code should be a 3 character ISO 4217 code. (default is USD).
  /// You can set the currency code for all events by calling the following method.
  void setCurrencyCode(String currencyCode) {
    _methodChannel
        .invokeMethod("setCurrencyCode", {'currencyCode': currencyCode});
  }

  /// Setting user webhook url to send web hook events.
  void setWebHookURL(String webHookURL) {
    _methodChannel.invokeMethod("setWebHookURL", {'webHookURL': webHookURL});
  }

  /// Setting your own customer ID enables you to cross-reference your own unique ID with AppCarry’s unique ID and the other devices’ IDs.
  /// This ID is available in AppCarry CSV reports along with Postback APIs for cross-referencing with your internal IDs.
  void setCustomerUserId(String id) {
    _methodChannel.invokeMethod("setCustomerUserId", {'id': id});
  }

  void setIsUpdate(bool isUpdate) {
    _methodChannel.invokeMethod("setIsUpdate", {'isUpdate': isUpdate});
  }

  /// Once this API is invoked, our SDK no longer communicates with our servers and stops functioning.
  /// In some extreme cases you might want to shut down all SDK activity due to legal and privacy compliance.
  /// This can be achieved with the stopTracking API.
  void stop(bool isStopped) {
    _methodChannel.invokeMethod("stop", {'isStopped': isStopped});
  }

  void enableLocationCollection(bool flag) {
    _methodChannel.invokeMethod("enableLocationCollection", {'flag': flag});
  }

  ///Please use updateServerUninstallToken instead
  @deprecated
  void enableUninstallTracking(String senderId) {
    print("Please use updateServerUninstallToken instead");
  }

  ///Manually pass the Firebase / GCM Device Token for Uninstall measurement.
  void updateServerUninstallToken(String token) {
    _methodChannel.invokeMethod("updateServerUninstallToken", {'token': token});
  }

  ///Set the user emails and encrypt them.
  void setUserEmails(List<String> emails, [EmailCryptType cryptType]) {
    if (cryptType != null) {
      int cryptTypeInt = EmailCryptType.values.indexOf(cryptType);
      _methodChannel.invokeMethod("setUserEmailsWithCryptType",
          {'emails': emails, 'cryptType': cryptTypeInt});
    } else {
      _methodChannel.invokeMethod("setUserEmails", {'emails': emails});
    }
  }

  ///Get AppCarry's unique device ID is created for every new install of an app.
  Future<String> getAppCarryUID() async {
    return await _methodChannel.invokeMethod("getAppCarryUID");
  }

  ///Set to true if you want to delay sdk init until CUID is set
  void waitForCustomerUserId(bool wait) {
    _methodChannel.invokeMethod("waitForCustomerUserId", {'wait': wait});
  }

  ///Returns `Stream`. Accessing AppCarry purchase validation data
  Stream<dynamic> validateAndLogInAppPurchase(
      String publicKey,
      String signature,
      String purchaseData,
      String price,
      String currency,
      Map<String, String> additionalParameters) {
    _methodChannel.invokeMethod("validateAndLogInAppPurchase", {
      'publicKey': publicKey,
      'signature': signature,
      'purchaseData': purchaseData,
      'price': price,
      'currency': currency,
      'additionalParameters': additionalParameters
    });
    return _registerValidatePurchaseCallback();
  }

  /// Set additional data to be sent to AppCarry.
  void setAdditionalData(Map<String, dynamic> customData) {
    _methodChannel
        .invokeMethod("setAdditionalData", {'customData': customData});
  }

  void _registerGCDListener() {
    _eventChannel.receiveBroadcastStream().listen((data) {
      var decodedJSON = jsonDecode(data);
      String type = decodedJSON['type'];
      switch (type) {
        case AppCarryConstants.AC_GET_CONVERSION_DATA:
          if (_acGCDStreamController != null &&
              !_acGCDStreamController.isClosed) {
            _acGCDStreamController.sink.add(decodedJSON);
          } else {
            if ((acOptions != null && acOptions.showDebug) ||
                (mapOptions != null &&
                    mapOptions[AppCarryConstants.AC_IS_DEBUG])) {
              print("GCD Stream controller is closed. the event wasn't sent");
            }
          }
          break;
        case AppCarryConstants.AC_ON_APP_OPEN_ATTRIBUTION:
          if (_acOpenAttributionStreamController != null &&
              !_acOpenAttributionStreamController.isClosed) {
            _acOpenAttributionStreamController.sink.add(decodedJSON);
          } else {
            if ((acOptions != null && acOptions.showDebug) ||
                (mapOptions != null &&
                    mapOptions[AppCarryConstants.AC_IS_DEBUG])) {
              print(
                  "OnAppOpenAttribution stream is closed. the event wasn't sent");
            }
          }
          break;
      }
    });
  }

  void _registerPurchaseValidateListener() {
    _eventChannel.receiveBroadcastStream().listen((data) {
      var decodedJSON = jsonDecode(data);
      String type = decodedJSON['type'];
      if (type == AppCarryConstants.AC_VALIDATE_PURCHASE) {
        _acValidtaPurchaseController.sink.add(decodedJSON);
      }
    });
  }

  ///The sharing filter blocks the sharing of S2S events via postbacks/API with integrated partners and other third-party integrations.
  ///Use the filter to fulfill regulatory requirements like GDPR and CCPA, to comply with user opt-out mechanisms, and for other business logic reasons.
  void setSharingFilter(List<String> filters) {
    _methodChannel.invokeMethod("setSharingFilter", filters);
  }

  void setSharingFilterForAllPartners() {
    _methodChannel.invokeMethod("setSharingFilterForAllPartners");
  }

  void generateInviteLink(
    AppCarryInviteLinkParams parameters,
    Function success,
    Function error,
  ) {
    Map<String, String> paramsMap;
    if (parameters != null) {
      paramsMap = _translateInviteLinkParamsToMap(parameters);
    }
    startListening(success, "successGenerateInviteLink");
    startListening(error, "errorGenerateInviteLink");
    _methodChannel.invokeMethod("generateInviteLink", paramsMap);
  }

  Map<String, String> _translateInviteLinkParamsToMap(
      AppCarryInviteLinkParams params) {
    Map<String, String> inviteLinkParamsMap = Map<String, String>();
    inviteLinkParamsMap['referrerImageUrl'] = params.referrerImageUrl;
    inviteLinkParamsMap['customerID'] = params.customerID;
    inviteLinkParamsMap['brandDomain'] = params.brandDomain;
    inviteLinkParamsMap['baseDeeplink'] = params.baseDeepLink;
    inviteLinkParamsMap['referrerName'] = params.referrerName;
    inviteLinkParamsMap['channel'] = params.channel;
    inviteLinkParamsMap['campaign'] = params.campaign;

    return inviteLinkParamsMap;
  }

  ///Set the inviteLinkTrackID ID that should be used for User-Invite-API.
  ///The link that is generated for the user invite will use this OneLink ID as the base link ID
  Future<void> setAppInviteLinkTrack(
      String inviteLinkTrackID, Function callback) async {
    startListening(callback, "successSetAppInviteLinkTrackID");
    await _methodChannel.invokeMethod("setAppInviteLinkTrackID", {
      'inviteLinkTrackID': inviteLinkTrackID,
    });
  }

  ///To attribute an impression use the following API call.
  ///Make sure to use the promoted App ID as it appears within the AppCarry dashboard.
  void logCrossPromotionImpression(String appId, String campaign, Map data) {
    _methodChannel.invokeMethod("logCrossPromotionImpression",
        {'appId': appId, 'campaign': campaign, 'data': data});
  }

  ///Use the following API to attribute the click and launch the app store's app page.
  void logCrossPromotionAndOpenStore(
      String appId, String campaign, Map params) {
    _methodChannel.invokeMethod("logCrossPromotionAndOpenStore", {
      'appId': appId,
      'campaign': campaign,
      'params': params,
    });
  }

  void setLinkTrackCustomDomain(List<String> brandDomains) {
    _methodChannel.invokeMethod("setLinkTrackCustomDomain", brandDomains);
  }
}
